@@ -1,6 +1,6 @@
 /*
  * Copyright 2015 by Andrea Vacondio (andrea.vacondio@gmail.com).
- * 
+ *
  * This file is part of the Sejda source code
  *
  * This program is free software: you can redistribute it and/or modify
@@ -19,14 +19,13 @@
 package org.sejda.impl.sambox.component;
 
 import static java.util.Optional.ofNullable;
-import static java.util.stream.Stream.concat;
-import static java.util.stream.Stream.of;
 import static org.sejda.impl.sambox.util.ViewerPreferencesUtils.getPageLayout;
 import static org.sejda.impl.sambox.util.ViewerPreferencesUtils.getPageMode;
 
 import java.awt.image.BufferedImage;
 import java.io.Closeable;
 import java.io.File;
+import java.io.FileOutputStream;
 import java.io.IOException;
 import java.util.Calendar;
 import java.util.HashSet;
@@ -36,6 +35,8 @@ import java.util.Set;
 import org.sejda.core.Sejda;
 import org.sejda.impl.sambox.util.FontUtils;
 import org.sejda.impl.sambox.util.PageLabelUtils;
+import org.sejda.model.encryption.EncryptionAtRestPolicy;
+import org.sejda.model.encryption.NoEncryptionAtRest;
 import org.sejda.model.exception.TaskException;
 import org.sejda.model.exception.TaskIOException;
 import org.sejda.model.image.ImageColorType;
@@ -65,7 +66,7 @@ import org.slf4j.LoggerFactory;
 
 /**
  * Wrapper over a {@link PDDocument}.
- * 
+ *
  * @author Andrea Vacondio
  */
 public class PDDocumentHandler implements Closeable {
@@ -82,7 +83,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Creates a new handler using the given document as underlying {@link PDDocument}.
-     * 
+     *
      * @param document
      */
     public PDDocumentHandler(PDDocument document) {
@@ -110,12 +111,14 @@ public class PDDocumentHandler implements Closeable {
      * set the creator on the underlying {@link PDDocument}
      */
     public void setCreatorOnPDDocument() {
-        document.getDocumentInformation().setCreator(Sejda.CREATOR);
+        if(!Sejda.CREATOR.isEmpty()) {
+            document.getDocumentInformation().setCreator(Sejda.CREATOR);
+        }
     }
 
     /**
      * Set the document information on the underlying {@link PDDocument}
-     * 
+     *
      * @param info
      */
     public void setDocumentInformation(PDDocumentInformation info) {
@@ -137,7 +140,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Sets the given page layout on the underlying {@link PDDocument}.
-     * 
+     *
      * @param layout
      */
     public void setPageLayoutOnDocument(PdfPageLayout layout) {
@@ -147,7 +150,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Sets the given page mode on the underlying {@link PDDocument}.
-     * 
+     *
      * @param mode
      */
     public void setPageModeOnDocument(PdfPageMode mode) {
@@ -157,7 +160,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Sets the page labels on the underlying {@link PDDocument}.
-     * 
+     *
      * @param labels
      */
     public void setPageLabelsOnDocument(Map<Integer, PdfPageLabel> labels) {
@@ -167,7 +170,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Sets the version on the underlying {@link PDDocument}.
-     * 
+     *
      * @param version
      */
     public void setVersionOnPDDocument(PdfVersion version) {
@@ -179,7 +182,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Adds the given {@link WriteOption}s to be used when the document is saved
-     * 
+     *
      * @param opts
      */
     public void addWriteOption(WriteOption... opts) {
@@ -190,7 +193,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Removes the given {@link WriteOption}s to be used when the document is saved
-     * 
+     *
      * @param opts
      */
     public void removeWriteOption(WriteOption... opts) {
@@ -233,30 +236,34 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Saves the underlying {@link PDDocument} to the given file.
-     * 
+     *
      * @param file
      * @throws TaskException
      */
-    public void savePDDocument(File file) throws TaskException {
-        savePDDocument(file, null);
+    public void savePDDocument(File file, EncryptionAtRestPolicy encryptionAtRestSecurity) throws TaskException {
+        savePDDocument(file, null, encryptionAtRestSecurity);
     }
 
     /**
      * Saves the underlying {@link PDDocument} to the given file and using the given standard security.
-     * 
+     *
      * @param file
      * @param security
      * @throws TaskException
      */
-    public void savePDDocument(File file, StandardSecurity security) throws TaskException {
+    public void savePDDocument(File file, StandardSecurity security, EncryptionAtRestPolicy encryptionAtRestSecurity)
+            throws TaskException {
         try {
             if (Boolean.getBoolean(SAMBOX_USE_ASYNC_WRITER)) {
-                LOG.trace("Saving document to {} using async writer", file);
+                this.addWriteOption(WriteOption.ASYNC_BODY_WRITE);
+            }
+            if (encryptionAtRestSecurity instanceof NoEncryptionAtRest) {
+                LOG.trace("Saving document to {} using options {}", file, writeOptions);
                 document.writeTo(file, security, writeOptions.stream().toArray(WriteOption[]::new));
             } else {
-                LOG.trace("Saving document to {}", file);
-                document.writeTo(file, security,
-                        concat(of(WriteOption.SYNC_BODY_WRITE), writeOptions.stream()).toArray(WriteOption[]::new));
+                LOG.trace("Saving document to {} using options {}", file, writeOptions);
+                document.writeTo(encryptionAtRestSecurity.encrypt(new FileOutputStream(file)), security,
+                        writeOptions.stream().toArray(WriteOption[]::new));
             }
         } catch (IOException e) {
             throw new TaskIOException("Unable to save to temporary file.", e);
@@ -277,7 +284,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Creates a copy of the given page and adds it to the underlying {@link PDDocument}
-     * 
+     *
      * @param page
      * @return The newly created page
      */
@@ -289,13 +296,15 @@ public class PDDocumentHandler implements Closeable {
 
         imported.setResources(page.getResources());
         imported.setRotation(page.getRotation());
+        // we don't retain the /Threads key in the Catalog so it doesn't make sense to keep /B
         imported.getCOSObject().removeItem(COSName.B);
+        imported.sanitizeDictionary();
         return addPage(imported);
     }
 
     /**
      * Adds the given page to the underlying {@link PDDocument}
-     * 
+     *
      * @param page
      * @return the page
      */
@@ -392,7 +401,7 @@ public class PDDocumentHandler implements Closeable {
 
     /**
      * Adds a blank page if the current total pages number is odd
-     * 
+     *
      * @param mediaBox
      *            media box size for the blank page
      * @return the added page or null if no page has been added
