@@ -1,6 +1,6 @@
 /*
  * Copyright 2015 by Andrea Vacondio (andrea.vacondio@gmail.com).
- * 
+ *
  * This file is part of the Sejda source code
  *
  * This program is free software: you can redistribute it and/or modify
@@ -18,35 +18,28 @@
  */
 package org.sejda.impl.sambox.util;
 
+import static java.util.Objects.isNull;
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
+import static java.util.stream.StreamSupport.stream;
+import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 import static org.sejda.sambox.util.BidiUtils.visualToLogical;
-import static org.sejda.util.RequireUtils.requireNotNullArg;
 
 import java.awt.geom.GeneralPath;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.Collections;
-import java.util.EnumMap;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
+import java.util.*;
 import java.util.regex.Pattern;
 
-import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.fontbox.ttf.TrueTypeFont;
-import org.sejda.fonts.OptionalUnicodeType0Font;
-import org.sejda.fonts.UnicodeType0Font;
 import org.sejda.impl.sambox.component.TextWithFont;
 import org.sejda.model.exception.TaskIOException;
-import org.sejda.model.pdf.FontResource;
+import org.sejda.model.exception.UnsupportedTextException;
 import org.sejda.model.pdf.StandardType1Font;
+import org.sejda.model.pdf.font.FontResource;
+import org.sejda.model.pdf.font.Type0FontsProvider;
 import org.sejda.sambox.cos.COSDictionary;
 import org.sejda.sambox.pdmodel.PDDocument;
 import org.sejda.sambox.pdmodel.common.PDRectangle;
@@ -99,6 +92,14 @@ public final class FontUtils {
 
     public static PDFont HELVETICA = PDType1Font.HELVETICA;
 
+    public static final FontResource[] TYPE0FONTS;
+
+    static {
+        TYPE0FONTS = stream(ServiceLoader.load(Type0FontsProvider.class).spliterator(), false)
+                .flatMap(p -> p.getFonts().stream()).sorted(Comparator.comparingInt(FontResource::priority))
+                .toArray(FontResource[]::new);
+    }
+
     /**
      * Mapping between Sejda and PDFBox standard type 1 fonts implementation
      *
@@ -147,8 +148,7 @@ public final class FontUtils {
             return docCache.get(font.getResource());
         }
 
-        InputStream in = font.getFontStream();
-        try {
+        try (InputStream in = font.getFontStream()) {
             PDType0Font loaded = PDType0Font.load(document, in);
             LOG.trace("Loaded font {}", loaded.getName());
             docCache.put(font.getResource(), loaded);
@@ -156,8 +156,6 @@ public final class FontUtils {
         } catch (IOException e) {
             LOG.warn("Failed to load font " + font, e);
             return null;
-        } finally {
-            IOUtils.closeQuietly(in);
         }
     }
 
@@ -167,24 +165,7 @@ public final class FontUtils {
      * @return a font capable of displaying the given string or null
      */
     public static final PDFont findFontFor(PDDocument document, String text) {
-        try {
-            // lets make sure the jar is in the classpath
-            Class.forName("org.sejda.fonts.UnicodeType0Font");
-            PDFont found = findFontAmong(document, text, UnicodeType0Font.values());
-            if (nonNull(found)) {
-                return found;
-            }
-            Class.forName("org.sejda.fonts.OptionalUnicodeType0Font");
-            return findFontAmong(document, text, OptionalUnicodeType0Font.values());
-
-        } catch (ClassNotFoundException clf) {
-            LOG.warn("Fallback fonts not available");
-        }
-        return null;
-    }
-
-    private static PDFont findFontAmong(PDDocument document, String text, FontResource... fonts) {
-        for (FontResource font : fonts) {
+        for (FontResource font : TYPE0FONTS) {
             PDFont loaded = loadFont(document, font);
             if (canDisplay(text, loaded)) {
                 LOG.debug("Found suitable font {} to display '{}'", loaded, text);
@@ -195,10 +176,8 @@ public final class FontUtils {
     }
 
     /**
-     * Check is given text contains only unicode whitespace characters
-     *
      * @param text
-     * @return
+     * @return true if given text contains only unicode whitespace characters
      */
     public static boolean isOnlyWhitespace(String text) {
         return text.replaceAll("\\p{Zs}", "").length() == 0;
@@ -208,7 +187,7 @@ public final class FontUtils {
      * Removes all unicode whitespace characters from the input string
      *
      * @param text
-     * @return
+     * @return the resulting string
      */
     public static String removeWhitespace(String text) {
         return text.replaceAll("\\p{Zs}", "").replaceAll("\\r\\n", "").replaceAll("\\n", "");
@@ -216,8 +195,11 @@ public final class FontUtils {
 
     public static boolean canDisplaySpace(PDFont font) {
         try {
+            // try encode
             font.encode(" ");
-            return true;
+
+            // see if width is non zero
+            return font.getStringWidth(" ") > 0;
         } catch (IllegalArgumentException | IOException | UnsupportedOperationException | NullPointerException e) {
             // Nope
         }
@@ -412,7 +394,7 @@ public final class FontUtils {
             PDDocument document) throws TaskIOException {
         List<String> lines = new ArrayList<>();
 
-        String label = org.sejda.core.support.util.StringUtils.normalizeWhitespace(rawLabel);
+        String label = org.sejda.commons.util.StringUtils.normalizeWhitespace(rawLabel);
 
         StringBuilder currentString = new StringBuilder();
         double currentWidth = 0;
@@ -421,9 +403,16 @@ public final class FontUtils {
 
         for (TextWithFont stringAndFont : resolvedStringsToFonts) {
             try {
+
                 PDFont resolvedFont = stringAndFont.getFont();
                 String resolvedLabel = stringAndFont.getText();
 
+                if (isNull(resolvedFont)) {
+                    throw new UnsupportedTextException(
+                            "Unable to find suitable font for string \"" + resolvedLabel + "\"",
+                            resolvedLabel);
+                }
+
                 String[] words = visualToLogical(resolvedLabel).split("(?<=\\b)");
                 for (String word : words) {
                     double textWidth = getSimpleStringWidth(word, resolvedFont, fontSize);
@@ -550,6 +539,41 @@ public final class FontUtils {
         return result;
     }
 
+    /**
+     * Splits an input string into multiple fragments, when glyphs with 0 width are detected
+     */
+    public static List<String> resolveTextFragments(String input, PDFont font) {
+        List<String> result = new ArrayList<>();
+        List<Integer> current = new ArrayList<>();
+        
+        for(int codePoint: input.codePoints().toArray()){
+            try {
+                if(font.getWidth(codePoint) == 0) {
+                    if(current.size() > 0) {
+                        StringBuilder s = new StringBuilder();
+                        current.stream().map(Character::toChars).forEach(s::append);
+                        result.add(s.toString());
+                    }
+                    
+                    result.add(new String(Character.toChars(codePoint)));
+                    current = new ArrayList<>();
+                } else {
+                    current.add(codePoint);
+                }
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        }
+
+        if(current.size() > 0) {
+            StringBuilder s = new StringBuilder();
+            current.stream().map(Character::toChars).forEach(s::append);
+            result.add(s.toString());
+        }
+        
+        return result;
+    }
+
     public static String removeUnsupportedCharacters(String text, PDDocument doc) {
         return replaceUnsupportedCharacters(text, doc, "");
     }
@@ -563,8 +587,13 @@ public final class FontUtils {
             }
         });
 
+        // replace unsupported groups of text longer ones first
+        // eg: first replace "ääç" and then "ä"
+        List<String> unsupportedSortedByLength = new ArrayList<>(unsupported);
+        unsupportedSortedByLength.sort((o1, o2) -> new Integer(o2.length()).compareTo(o1.length()));
+
         String result = text;
-        for (String s : unsupported) {
+        for (String s : unsupportedSortedByLength) {
             result = result.replaceAll(Pattern.quote(s), replacement);
         }
 
