@@ -20,7 +20,13 @@ package org.sejda.impl.sambox;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.Matchers.containsString;
-import static org.junit.Assert.*;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNull;
+import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.sejda.TestUtils.encryptedAtRest;
 import static org.sejda.core.service.TestUtils.assertPageLabelIndexesAre;
 import static org.sejda.core.service.TestUtils.assertPageLabelRangeIs;
 
@@ -39,6 +45,7 @@ import org.sejda.core.notification.context.ThreadLocalNotificationContext;
 import org.sejda.core.service.BaseTaskTest;
 import org.sejda.impl.sambox.component.DocBuilder;
 import org.sejda.impl.sambox.component.PdfTextExtractorByArea;
+import org.sejda.model.exception.InvalidTaskParametersException;
 import org.sejda.model.exception.TaskIOException;
 import org.sejda.model.input.ImageMergeInput;
 import org.sejda.model.input.MergeInput;
@@ -184,6 +191,14 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
         doExecuteMergeAll(false, 23, parameters);
     }
 
+    @Test
+    public void executeMergeAllTocNamesNoFont() throws IOException {
+        MergeParameters parameters = setUpParameters(getInput());
+        parameters.addInput(new PdfMergeInput(customInput("pdf/with_meta.pdf", "հայերէն.pdf")));
+        parameters.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
+        doExecuteMergeAll(false, 18, parameters);
+    }
+
     @Test
     public void executeMergeAllWithOutlineDiscardingOutline() throws IOException {
         MergeParameters parameters = setUpParameters(getInputWithOutline());
@@ -509,6 +524,23 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
         });
     }
 
+    @Test
+    public void pageFooterAndTocwithDocTitles() throws IOException {
+        List<PdfMergeInput> input = new ArrayList<PdfMergeInput>();
+        input.add(new PdfMergeInput(regularInput()));
+        input.add(new PdfMergeInput(customInput("pdf/test_file.pdf", "my_file.pdf")));
+        MergeParameters parameters = setUpParameters(input);
+        parameters.setTableOfContentsPolicy(ToCPolicy.DOC_TITLES);
+        parameters.setFilenameFooter(true);
+        testContext.pdfOutputTo(parameters);
+        execute(parameters);
+        testContext.assertTaskCompleted();
+        testContext.assertPages(16).forEachPdfOutput(d -> {
+            assertFooterHasText(d.getPage(1), "test-file 2");
+            assertFooterHasText(d.getPage(12), "my_file 13");
+        });
+    }
+
     @Test
     public void pageFooterAndTocAddBlank() throws IOException {
         MergeParameters parameters = setUpParameters(getInput());
@@ -624,6 +656,20 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
         testContext.assertTaskFailed("An error occurred creating PDImageXObject from file source: corrupt.png");
     }
 
+    @Test
+    public void mergeImagesWithValidation() throws IOException {
+        MergeParameters parameters = new MergeParameters();
+        parameters.setLenient(false);
+        parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
+        ImageMergeInput input1 = new ImageMergeInput(customNonPdfInput("image/draft.png", "draft.png"));
+        input1.setPageSize(null);
+        parameters.addInput(input1);
+
+        testContext.pdfOutputTo(parameters);
+        executeWithValidation(parameters);
+        testContext.assertTaskFailed(InvalidTaskParametersException.class);
+    }
+
     @Test
     public void mergeWithUnreadableImageWarning() throws IOException {
         MergeParameters parameters = new MergeParameters();
@@ -644,12 +690,8 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
     public void mergeKeepingPageLabels() throws IOException {
         MergeParameters parameters = new MergeParameters();
         parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
-        PDDocument doc1 = new DocBuilder().withPages(2)
-                .withPageLabelRange(0, "r", "Intro:")
-                .get();
-        PDDocument doc2 = new DocBuilder().withPages(3)
-                .withPageLabelRange(0, "D")
-                .get();
+        PDDocument doc1 = new DocBuilder().withPages(2).withPageLabelRange(0, "r", "Intro:").get();
+        PDDocument doc2 = new DocBuilder().withPages(3).withPageLabelRange(0, "D").get();
         parameters.addInput(new PdfMergeInput(customInput(doc1, "doc1.pdf")));
         parameters.addInput(new PdfMergeInput(customInput(doc2, "doc2.pdf")));
         parameters.setCatalogPageLabelsPolicy(CatalogPageLabelsPolicy.RETAIN);
@@ -673,12 +715,8 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
     public void mergeKeepingPageLabelsButDiscardingDecimalsStart() throws IOException {
         MergeParameters parameters = new MergeParameters();
         parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
-        PDDocument doc1 = new DocBuilder().withPages(2)
-                .withPageLabelRange(0, "D", null, 2)
-                .get();
-        PDDocument doc2 = new DocBuilder().withPages(3)
-                .withPageLabelRange(0, "D", null, 7)
-                .get();
+        PDDocument doc1 = new DocBuilder().withPages(2).withPageLabelRange(0, "D", null, 2).get();
+        PDDocument doc2 = new DocBuilder().withPages(3).withPageLabelRange(0, "D", null, 7).get();
 
         parameters.addInput(new PdfMergeInput(customInput(doc1, "doc1.pdf")));
         parameters.addInput(new PdfMergeInput(customInput(doc2, "doc2.pdf")));
@@ -704,12 +742,8 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
         MergeParameters parameters = new MergeParameters();
         parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
 
-        PDDocument doc1 = new DocBuilder().withPages(2)
-                .withPageLabelRange(0, "r", "Intro:")
-                .get();
-        PDDocument doc2 = new DocBuilder().withPages(3)
-                .withPageLabelRange(0, "D")
-                .get();
+        PDDocument doc1 = new DocBuilder().withPages(2).withPageLabelRange(0, "r", "Intro:").get();
+        PDDocument doc2 = new DocBuilder().withPages(3).withPageLabelRange(0, "D").get();
 
         parameters.addInput(new PdfMergeInput(customInput(doc1, "doc1.pdf")));
         parameters.addInput(new PdfMergeInput(customInput(doc2, "doc2.pdf")));
@@ -812,6 +846,22 @@ public class MergeSamboxTaskTest extends BaseTaskTest<MergeParameters> {
         });
     }
 
+    @Test
+    public void atRestEncryptionTest() throws IOException {
+        MergeParameters parameters = new MergeParameters();
+        parameters.setExistingOutputPolicy(ExistingOutputPolicy.OVERWRITE);
+        parameters.addInput(new ImageMergeInput(encryptedAtRest(customNonPdfInput("image/draft.png"))));
+        parameters.addInput(new ImageMergeInput(encryptedAtRest(customNonPdfInput("image/draft.tiff"))));
+        parameters.addInput(new PdfMergeInput(encryptedAtRest(customInput("pdf/test-pdf.pdf"))));
+
+        testContext.pdfOutputTo(parameters);
+        execute(parameters);
+
+        testContext.assertTaskCompleted();
+        testContext.assertNoTaskWarnings();
+        testContext.assertPages(13);
+    }
+
     private float widthOfCropBox(PDPage page) {
         return page.getCropBox().rotate(page.getRotation()).getWidth();
     }
