@@ -19,10 +19,12 @@
 package org.sejda.impl.sambox.component;
 
 import static org.hamcrest.Matchers.containsString;
+import static org.hamcrest.Matchers.instanceOf;
 import static org.hamcrest.Matchers.is;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertNotNull;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.mock;
 
@@ -33,6 +35,7 @@ import org.junit.Test;
 import org.sejda.core.service.TestUtils;
 import org.sejda.io.SeekableSources;
 import org.sejda.model.exception.TaskException;
+import org.sejda.model.exception.UnsupportedTextException;
 import org.sejda.model.input.PdfFileSource;
 import org.sejda.model.input.PdfMergeInput;
 import org.sejda.model.parameter.MergeParameters;
@@ -42,6 +45,7 @@ import org.sejda.sambox.pdmodel.PDDocument;
 import org.sejda.sambox.pdmodel.PDPage;
 import org.sejda.sambox.pdmodel.common.PDRectangle;
 import org.sejda.sambox.pdmodel.interactive.annotation.PDAnnotationLink;
+import org.sejda.sambox.pdmodel.interactive.documentnavigation.destination.PDPageXYZDestination;
 import org.sejda.sambox.text.PDFTextStripper;
 
 /**
@@ -306,16 +310,17 @@ public class TableOfContentsCreatorTest {
         PDPage pageDest1 = new PDPage(), pageDest2 = new PDPage(), pageDest3 = new PDPage();
         TableOfContentsCreator victim = new TableOfContentsCreator(params, doc);
         victim.appendItem("This is item 1", 1, pageDest1);
-        victim.appendItem("This is item 2 that has a very long name and should not be truncated so that the version is visible at the end v7.pdf", 10, pageDest2);
+        victim.appendItem(
+                "This is item 2 that has a very long name and should not be truncated so that the version is visible at the end v7.pdf",
+                10, pageDest2);
         victim.appendItem("This is item 3", 14, pageDest3);
         victim.pageSizeIfNotSet(PDRectangle.A4);
         victim.addToC();
 
         TestUtils.assertPageTextExactLines(doc.getPage(0),
-                "This is item 1   1\n" +
-                "This is item 2 that has a very long name and should not be\n" +
-                "truncated so that the version is visible at the end v7.pdf   10\n" +
-                "This is item 3   14\n");
+                "This is item 1   1\n" + "This is item 2 that has a very long name and should not be\n"
+                        + "truncated so that the version is visible at the end v7.pdf   10\n"
+                        + "This is item 3   14\n");
 
         // verify size of the clickable annotations on top of the TOC items
         List<PDAnnotationLink> annotations = TestUtils.getAnnotationsOf(doc.getPage(0), PDAnnotationLink.class);
@@ -335,44 +340,59 @@ public class TableOfContentsCreatorTest {
     }
 
     @Test
-    public void test_Toc_Long_Item_That_Wraps_At_The_End_Of_The_Page() throws IOException, TaskException {
+    public void test_Toc_Long_Item_That_Wraps_At_The_End_Of_The_Page() throws TaskException {
         MergeParameters params = new MergeParameters();
         params.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
         PDDocument doc = new PDDocument();
         TableOfContentsCreator victim = new TableOfContentsCreator(params, doc);
-        for(int i = 0; i < 30; i++) {
+        for (int i = 0; i < 30; i++) {
             victim.appendItem("This is an item", 1, new PDPage());
         }
 
-        victim.appendItem("This is a long item that has a very long name and should not be truncated so that the version is visible at the end v7.pdf", 10, new PDPage());
+        victim.appendItem(
+                "This is a long item that has a very long name and should not be truncated so that the version is visible at the end v7.pdf",
+                10, new PDPage());
         victim.appendItem("This is an item", 14, new PDPage());
         victim.pageSizeIfNotSet(PDRectangle.A4);
         victim.addToC();
 
-        TestUtils.assertPageTextDoesNotContain(doc.getPage(0),
-                "This is a long item that");
+        TestUtils.assertPageTextDoesNotContain(doc.getPage(0), "This is a long item that");
 
-        TestUtils.assertPageTextContains(doc.getPage(1),
-                "This is a long item that");
+        TestUtils.assertPageTextContains(doc.getPage(1), "This is a long item that");
     }
 
     @Test
-    public void test_Toc_Long_Item_That_Has_No_Word_Breaks() throws IOException, TaskException {
+    public void test_Toc_Long_Item_That_Has_No_Word_Breaks() throws TaskException {
         MergeParameters params = new MergeParameters();
         params.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
         PDDocument doc = new PDDocument();
         TableOfContentsCreator victim = new TableOfContentsCreator(params, doc);
-        victim.appendItem("This_is_a_file_that_has_a_very_long_name_and_should_not_be_truncated_so_that_the_version_is_visible_at_the_end_v7.pdf", 10, new PDPage());
+        victim.appendItem(
+                "This_is_a_file_that_has_a_very_long_name_and_should_not_be_truncated_so_that_the_version_is_visible_at_the_end_v7.pdf",
+                10, new PDPage());
         victim.pageSizeIfNotSet(PDRectangle.A4);
         victim.addToC();
 
         TestUtils.assertPageTextExactLines(doc.getPage(0),
-                "This_is_a_file_that_has_a_very_long_name_and_should_not_be_tr-\n" +
-                        "uncated_so_that_the_version_is_visible_at_the_end_v7.pdf   10\n");
+                "This_is_a_file_that_has_a_very_long_name_and_should_not_be_tr-\n"
+                        + "uncated_so_that_the_version_is_visible_at_the_end_v7.pdf   10\n");
     }
 
     @Test
-    public void test_Toc_Item_Requiring_Multiple_Fonts() throws IOException, TaskException {
+    public void test_Toc_Item_Requiring_Multiple_Fonts() throws TaskException {
+        MergeParameters params = new MergeParameters();
+        params.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
+        PDDocument doc = new PDDocument();
+        TableOfContentsCreator victim = new TableOfContentsCreator(params, doc);
+        victim.appendItem("Item multiple fonts ทดสอบ", 10, new PDPage());
+        victim.pageSizeIfNotSet(PDRectangle.A4);
+        victim.addToC();
+
+        TestUtils.assertPageTextExactLines(doc.getPage(0), "Item multiple fonts ทดสอบ   10\n");
+    }
+
+    @Test(expected = UnsupportedTextException.class)
+    public void tocItemsMultipleFontsButNotFound() throws TaskException {
         MergeParameters params = new MergeParameters();
         params.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
         PDDocument doc = new PDDocument();
@@ -380,8 +400,6 @@ public class TableOfContentsCreatorTest {
         victim.appendItem("Item multiple fonts հայերէն", 10, new PDPage());
         victim.pageSizeIfNotSet(PDRectangle.A4);
         victim.addToC();
-
-        TestUtils.assertPageTextExactLines(doc.getPage(0), "Item multiple fonts հայերէն   10\n");
     }
 
     @Test
@@ -392,7 +410,8 @@ public class TableOfContentsCreatorTest {
 
         PDDocument doc = new PDDocument();
         PDPage pageA = new PDPage(), pageB = new PDPage();
-        doc.addPage(pageA); doc.addPage(pageB);
+        doc.addPage(pageA);
+        doc.addPage(pageB);
 
         PageTextWriter.writeHeader(doc, pageA, "PageA");
         PageTextWriter.writeHeader(doc, pageB, "PageB");
@@ -414,4 +433,50 @@ public class TableOfContentsCreatorTest {
         // blank page pageB
         TestUtils.assertPageTextExactLines(doc.getPage(3), "PageB\n");
     }
+
+    @Test
+    public void tocItemsLinkWithRotatedPages() throws TaskException, IOException {
+        MergeParameters params = new MergeParameters();
+        params.setTableOfContentsPolicy(ToCPolicy.FILE_NAMES);
+        PDDocument doc = new PDDocument();
+        PDPage page = new PDPage(PDRectangle.A4);
+        PDPage page90 = new PDPage(PDRectangle.A4);
+        page90.setRotation(90);
+        PDPage page180 = new PDPage(PDRectangle.A4);
+        page180.setRotation(180);
+        PDPage page270 = new PDPage(PDRectangle.A4);
+        page270.setRotation(270);
+        TableOfContentsCreator victim = new TableOfContentsCreator(params, doc);
+        victim.appendItem("Item ", 1, page);
+        victim.appendItem("Item 90", 2, page90);
+        victim.appendItem("Item 180", 3, page180);
+        victim.appendItem("Item 270", 4, page270);
+        victim.addToC();
+
+        List<PDAnnotationLink> annotations = TestUtils.getAnnotationsOf(doc.getPage(0), PDAnnotationLink.class);
+        PDAnnotationLink link = annotations.get(0);
+        assertNotNull(link);
+        assertThat(link.getDestination(), instanceOf(PDPageXYZDestination.class));
+        assertEquals(0, ((PDPageXYZDestination) link.getDestination()).getLeft());
+        assertEquals((int) PDRectangle.A4.getHeight(), ((PDPageXYZDestination) link.getDestination()).getTop());
+
+        PDAnnotationLink link90 = annotations.get(1);
+        assertNotNull(link90);
+        assertThat(link90.getDestination(), instanceOf(PDPageXYZDestination.class));
+        assertEquals(0, ((PDPageXYZDestination) link90.getDestination()).getLeft());
+        assertEquals(0, ((PDPageXYZDestination) link90.getDestination()).getTop());
+
+        PDAnnotationLink link180 = annotations.get(2);
+        assertNotNull(link180);
+        assertThat(link180.getDestination(), instanceOf(PDPageXYZDestination.class));
+        assertEquals((int) PDRectangle.A4.getWidth(), ((PDPageXYZDestination) link180.getDestination()).getLeft());
+        assertEquals(0, ((PDPageXYZDestination) link180.getDestination()).getTop());
+
+        PDAnnotationLink link270 = annotations.get(3);
+        assertNotNull(link270);
+        assertThat(link270.getDestination(), instanceOf(PDPageXYZDestination.class));
+        assertEquals((int) PDRectangle.A4.getWidth(), ((PDPageXYZDestination) link270.getDestination()).getLeft());
+        assertEquals((int) PDRectangle.A4.getHeight(), ((PDPageXYZDestination) link270.getDestination()).getTop());
+
+    }
 }
