@@ -18,6 +18,7 @@
  */
 package org.sejda.impl.sambox.component.optimization;
 
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.mock;
@@ -119,8 +120,25 @@ public class ResourcesHitterTest {
     public void exceptionDoesntBubble() {
         PDPage page = mock(PDPage.class);
         when(page.getCropBox()).thenReturn(new PDRectangle(2f, 2f));
-        doThrow(IOException.class).when(page).getResources();
+        doThrow(RuntimeException.class).when(page).getResources();
         victim.accept(page);
     }
 
+    @Test
+    public void sameFontSameInUseFontInstance() throws IOException {
+        try (PDDocument document = PDFParser.parse(SeekableSources.inMemorySeekableSourceFrom(
+                getClass().getClassLoader().getResourceAsStream("pdf/multiple_res_dic_sharing_same_font.pdf")))) {
+            document.getPages().forEach(victim::accept);
+            PDPage page0 = document.getPage(0);
+            InUseFontDictionary page0Font = page0.getResources().getCOSObject()
+                    .getDictionaryObject(COSName.FONT, COSDictionary.class)
+                    .getDictionaryObject(COSName.getPDFName("F1"), InUseFontDictionary.class);
+            PDPage page1 = document.getPage(1);
+            InUseFontDictionary page1Font = page1.getResources().getCOSObject()
+                    .getDictionaryObject(COSName.FONT, COSDictionary.class)
+                    .getDictionaryObject(COSName.getPDFName("F1"), InUseFontDictionary.class);
+            assertEquals(page0Font, page1Font);
+        }
+    }
+
 }
